@@ -1,6 +1,7 @@
 from authentication import Authentication
 from authentication import NoAuthentication
 from authentication import SessionAuthentication
+from preserialize import serialize as preserializer
 import serializers 
 import deserializers
 import exceptions
@@ -11,7 +12,6 @@ from django.core.exceptions import ImproperlyConfigured
 from django.core.exceptions import ValidationError
 from django.core.exceptions import NON_FIELD_ERRORS
 from django.core.paginator import Paginator, EmptyPage, PageNotAnInteger
-from preserialize import serialize as preserializer
 
 
 class HandlerMetaClass(type):
@@ -50,38 +50,12 @@ class HandlerMetaClass(type):
 class HandlerControlFlow(object):
     """
     This class describes the basic control flow and outer shell operations of any handler. 
-    Basically the generic behavior is set here, whereas more specific behavior
-    that corresponds to HTTP methods are  described in its subclasses.
+    Basically the very generic high-level behavior layout is defined here. Its
+    more fine-grained parts are defined in classes ``BaseHandler`` and
+    ``ModelHandler``.
     
-    Don't subclass it.
+    Don't subclass directly in a concrete handler class.
     """
-    __metaclass__ = HandlerMetaClass
-
-    # Override to define the handler's output representation. It should follow
-    # the syntax of ``django-preserialize`` templates.
-    # See <https://github.com/bruth/django-preserialize#conventions>
-    template = {}     
-
-    # List of allowed HTTP methods.
-    # GET, POST, PUT, PLURAL_PUT, DELETE, PLURAL_DELETE
-    http_methods = []
-
-    # authentication method: Should be an instance of any of the classes in the
-    # ``authentication`` module, other than ``Authentication``
-    authentication = None
-
-    # Allowed request body fields for POST and PUT requests
-    post_body_fields = put_body_fields = []    
-
-    # Filters, declared as strings. Each string indicates the method name of
-    # each filter. Every method's signature is ``(self, data, request, *args, **kwargs)``
-    # The method should define all its logic, and return a subset of ``data``.
-    # This generic scheme, requires that we write some code for every filter
-    # method, but is very flexible and powerful.
-    filters = []
-
-    # Default item per page, when pagination is requested
-    items_per_page = 10
 
     def dispatch(self):
         """
@@ -95,14 +69,21 @@ class HandlerControlFlow(object):
         try:
             self.preprocess()
             data, pagination = self.process()
-            response_data, headers = self.postprocess(
+            dic = self.postprocess(
                 data, pagination,
             )
-        except Exception, e:
-            # If exception, return the appropriate http.HttpResponse object
-            return exceptions.handle_exception(e, self.request)
 
-        return self.response(response_data, headers)
+        except Exception, e:
+            # handle exception
+            response, headers = self.handle_exception(e)
+
+        else:
+            serialized_data, headers = serializers.serialize_response_data(
+                dic, self.request, self.args, self.kwargs
+            ) 
+            response = http.HttpResponse(serialized_data)            
+
+        return self.patch_response(response, headers)
 
     def preprocess(self):
         """
@@ -161,10 +142,7 @@ class HandlerControlFlow(object):
             data: Result of operation
             pagination: Dictionary with pagination data
         Returns:
-            (serialized, headers): ``serialized`` is the
-            serialized response, ready to be passed to the HTTPResponse object, and
-            ``headers`` is a dictionary of headers to be passed to the HTTPResponse
-            object.
+            Whole response data dictionary
         """
         self.inject_data_hook(data)
         # Serialize ``data`` to python data structures
@@ -172,7 +150,7 @@ class HandlerControlFlow(object):
         # finalize any pending data processing
         self.finalize_pending(data)
         # Package the python_data to a dictionary
-        pack = self.package(python_data, pagination)
+        return self.package(python_data, pagination)
         # Return serialized response plus any http headers, like
         # ``content-type`` that need to be passed in the HttpResponse instance.
         serialized, headers = serializers.serialize_response_data(
@@ -181,6 +159,58 @@ class HandlerControlFlow(object):
         
         return serialized, headers
 
+    def handle_exception(self, e):
+        """
+        Invoked by ``dispatch``
+        Handles any exceptions that might have been raised
+
+        Args:
+            e: Exception instance
+        Returns:
+            (http_response, headers): ``http_response`` is an HttpResponse
+            instance. ``headers`` is a dictionary of key-value pairs to be used
+            as headers on the HttpResponse object.
+        """
+        if isinstance(e, exceptions.APIException):
+            return e.get_http_response_and_headers()
+        else:
+            exc = exceptions.OtherException(self.request)
+            return exc.get_http_response_and_headers()
+
+
+
+class BaseHandler(HandlerControlFlow):
+    """
+    This class describes a Base handler's operation.
+    """
+    __metaclass__ = HandlerMetaClass
+
+    # Override to define the handler's output representation. It should follow
+    # the syntax of ``django-preserialize`` templates.
+    # See <https://github.com/bruth/django-preserialize#conventions>
+    template = {}     
+
+    # List of allowed HTTP methods.
+    # GET, POST, PUT, PLURAL_PUT, DELETE, PLURAL_DELETE
+    http_methods = []
+
+    # authentication method: Should be an instance of any of the classes in the
+    # ``authentication`` module, other than ``Authentication``
+    authentication = None
+
+    # Allowed request body fields for POST and PUT requests
+    post_body_fields = put_body_fields = []    
+
+    # Filters, declared as strings. Each string indicates the method name of
+    # each filter. Every method's signature is ``(self, data, request, *args, **kwargs)``
+    # The method should define all its logic, and return a subset of ``data``.
+    # This generic scheme, requires that we write some code for every filter
+    # method, but is very flexible and powerful.
+    filters = []
+
+    # Default item per page, when pagination is requested
+    items_per_page = 10
+    
     def authentication_hook(self):
         """
         Invoked by ``preprocess``
@@ -222,7 +252,9 @@ class HandlerControlFlow(object):
             exceptions.BadRequest: If request body is not valid according to content-type
         """
         try:
-            self.request.data = deserializers.deserialize_request_body(self.request, *self.args, **self.kwargs)
+            self.request.data = deserializers.deserialize_request_body(
+                self.request, *self.args, **self.kwargs
+            )
         except (exceptions.UnsupportedMediaType, exceptions.BadRequest):
             raise
 
@@ -352,9 +384,10 @@ class HandlerControlFlow(object):
             'query_log': connection.queries,
         }
 
-    def response(self, data, headers={}):
+    def patch_response(self, response, headers={}):
         """
-        Invoked by ``dispatch``
+        Invoked by ``dispatch``. Wraps data and headers in an HttpResponse
+        object. Override to add more headers or modify any response attribute.
 
         Args:
             data: Data serialized in text
@@ -363,16 +396,10 @@ class HandlerControlFlow(object):
         Returns:
             http.HttpResponse object.
         """
-        res = http.HttpResponse(data)
         for key, value in headers.items():
-            res[key] = value
-        return res
+            response[key] = value
+        return response
 
-
-class BaseHandler(HandlerControlFlow):
-    """
-    This class describes a base handler's real operation.
-    """
     def get(self):
         """
         Invoked by ``dispatch``.
@@ -555,6 +582,7 @@ class ModelHandler(BaseHandler):
     """
     This class describes a Model handler's operation.
     """
+
     # Override to define the handler's model
     model = None
 
